@@ -14,19 +14,21 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+    MA 02110-1301 USA.
 */
 
 #ifndef LIB_SENSORS_CONF_H
 #define LIB_SENSORS_CONF_H
 
 /* This is defined in conf-lex.l */
-extern int sensors_yylex(void);
+int sensors_yylex(void);
 extern char sensors_lex_error[];
+extern const char *sensors_yyfilename;
 extern int sensors_yylineno;
 extern FILE *sensors_yyin;
 
 /* This is defined in conf-parse.y */
-extern int sensors_yyparse(void);
+int sensors_yyparse(void);
 
 #endif /* LIB_SENSORS_CONF_H */
