@@ -21,7 +21,7 @@
 #ifndef LIB_SENSORS_SCANNER_H
 #define LIB_SENSORS_SCANNER_H
 
-int sensors_scanner_init(FILE *input);
+int sensors_scanner_init(FILE *input, const char *filename);
 void sensors_scanner_exit(void);
 
 #endif
